// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class _InvertingComparable implements Comparable {
  Comparable delegate;

  _InvertingComparable(this.delegate);

  int compareTo(Comparable other) {
    if (other is! _InvertingComparable) {
      throw new ExpectException("_InvertingComparable can only compare with other _InvertingComparable");
    }

    return -1 * delegate.compareTo(other.delegate);
  }
}

class _CompoundComparable implements Comparable {
  List<Comparable> delegates;

  _CompoundComparable(List<Comparable> this.delegates);

  int compareTo(Comparable other) {
    if (other is! _CompoundComparable) {
      throw new ExpectException("_CompoundComparable can only compare with other _CompoundComparable");
    }
    if (delegates.length != other.delegates.length) {
      throw new ExpectException("Both _CompoundComparables must have same length");
    }

    for (int i = 0; i < delegates.length; i++) {
      var comparison = delegates[i].compareTo(other.delegates[i]);
      if (comparison != 0) {
        return comparison;
      }
    }
    return 0;
  }
}

/// Returns the given [Comparable] as is. Useful for `Query.sortBy`.
Comparable asc(Comparable c) => c;
/// Returns a [Comparable] that is an inversion of [c].
/// Useful for `Query.sortBy`.
Comparable desc(Comparable c) => new _InvertingComparable(c);

